SystemOrganization addCategory: #'Launcher-Core'!

!MCWorkingCopy class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:18'!
launchFrom: launcher

^launcher launchReportFor: self ! !

!MCWorkingCopy class methodsFor: '*launcher-core' stamp: 'kph 2/13/2007 16:56'!
workingCopies
	^ self allManagers asSortedCollection:
		[ :a :b | a package name <= b package name ]! !

!MCWorkingCopy methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 11:54'!
printOn: stream

super printOn: stream.
stream space.
self description printOn: stream.! !

Object subclass: #Launcher
	instanceVariableNames: 'early nextParameterIndex image launchClass nextLaunchDocument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Launcher-Core'!

!Launcher commentStamp: 'kph 2/15/2007 23:59' prior: 0!
Launcher is an adaptation of the launch process which is intended to be supported by a refactoring of the AutoStart infrastructure. This release integrates with the current AutoStart infrastructure.

The intention is to enable any class to participate in the launching process without registering in advance. Each class may choose how its parameters should be parsed and when in the launching process it is appropriate to run.

Classes implement launchFrom: to participate which supplies a handle to the launcher. The class may then request parameters that are relevent to the start time of choice and using the parsing scheme of choice from the launcher.

The launcher may also provide other common services. see #launchReportFor: 

two options currently available are
	launcher startParameters "for starting before AutoStart"
	launcher launchParameters "for starting during AutoStart"

with 
	launcher launchParametersOldStyle available if preferred.

Packaging:

At present all Launch functionality is included in this package including the launching functionality added to other classes, such as seaside's WAKom, and Installer.!

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:13'!
classProjectLauncher

^Smalltalk at: #ProjectLauncher ifAbsent: [ self error: 'ProjectLauncher not present' ]! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:27'!
initialize

	"Establish whether we are running as part of the squeak initialization process"

	Smalltalk garbageCollect.
	self classProjectLauncher allInstances size = 0 ifTrue: [ ^ self ].

	"we be our own launcher this time"

	Smalltalk at: #WorldState ifPresent: [ :ws | 
		ws addDeferredUIMessage: [ self new beFirstTimeLauncher initiateStartingSequence ] fixTemps
	].
	Smalltalk at: #WorldState ifAbsent: [  [ self new beFirstTimeLauncher initiateStartingSequence ] fixTemps fork ].

	"for the future"
	
	AutoStart addLauncherFirst: self.
	Smalltalk addToStartUpList: self after: SecurityManager.	
	! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 06:47'!
launchFrom: launcher

	^self launchWith: launcher launchParameters			! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 09:51'!
launchWith: params

params at: 'TEST' ifPresent: [ :v |
	Transcript cr; show: 'HERE'; cr
].

params at: 'SCRIPTS' ifPresent: [:v | 

	 CodeLoader new loadSourceFiles: (Array with: (v findTokens: ' ')).

].

^true! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:26'!
newLauncher

^self new beLauncher! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:18'!
newStarter

^self new beStarter! !

!Launcher class methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:19'!
startUp: resuming

"we are sent this when starting before autostart"

self newStarter
 launchImage: SmalltalkImage current withClass: self 
! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:24'!
beFirstTimeLauncher

nextParameterIndex := 3.
early := false! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
beLauncher

early := false! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
beStarter

early := true! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 18:20'!
classes

^ Smalltalk! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:23'!
initialize

nextParameterIndex := 2! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:25'!
initiateStartingSequence

 ^self launchImage: SmalltalkImage current withClass: self class	! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
isLaunch

^self isStart not! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:14'!
isStart

^early ifNil: [ false ]! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/14/2007 07:53'!
launchImage: imageOrMock withClass: theFirstLauncherClass  

	image := imageOrMock.
	nextLaunchDocument := nil.
	launchClass := theFirstLauncherClass.
	
	[ launchClass launchFrom: self ] 
		whileTrue: [ 
			launchClass := self nextClassToLaunch.
			launchClass ifNil: [ ^self ] ].
	
	  
			
			
			
			! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
launchParameters

^self isLaunch ifTrue: [ self nextParametersNewStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
launchParametersOldStyle

^self isLaunch ifTrue: [ self nextParametersOldStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'reporting' stamp: 'kph 2/15/2007 06:14'!
launchReportFor: target
 
 | params reports dest quit |

 params := self launchParameters.

 reports := params at: 'REPORT' ifAbsent: [ ^ self error: 'no report requested' ].

 reports := reports findTokens: ' '.

 dest := params at: 'TO' ifAbsent:  (self stdout).
 
 reports do: [ : reportSelector |
 	dest isString 
		ifTrue: [ CrLfFileStream forceNewFileNamed: (dest copyReplaceAll:'*' with: reportSelector) 
					do: [ :file |  self report: reportSelector asSymbol for: target on: file ]     ]
		ifFalse: [ self report: reportSelector asSymbol for: target on: dest ].
 ].

 quit := params at: 'QUIT' ifAbsent: [ false ].
 quit ~= false ifTrue: [ SmalltalkImage current snapshot: false andQuit: true ].

 ^true! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 22:18'!
nextClassToLaunch	
			
	^nextLaunchDocument ifNotNilDo: [:l | self classes at: l asSymbol ifAbsent: [ nil ] ]
	
 ! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 21:44'!
nextParametersNewStyle

	"this utility extracts the unprocessed parameters into a dictionary from the typical form:
	
	param1=true param2=''hello world'' +param3 -param4

	i.e. items with an = are parameters,
	+ gives param3=true
	- give param4=false

	items without +=- are considered to refer to the next Launch Class or document.
	"

	| key value parameters item |
	
	parameters := Dictionary new.
	
	parameters at: #launcher put: launchClass.
 
	[ (item := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			item := item readStream.
			value := nil.
			(item peek = $-) ifTrue: [ value := false. item next  ].
			(item peek = $+) ifTrue: [ value := true.  item next  ].
			key := item upTo: $=.
			value isNil & item atEnd not ifTrue: [ value := item upToEnd ].
			value 
				ifNotNil: [ parameters at: key asUppercase put: value ] 
				ifNil: [ nextLaunchDocument := key. 
					    ^parameters ].
	].
	
	nextLaunchDocument := nil.
	^parameters

			
			
			
			! !

!Launcher methodsFor: 'as yet unclassified' stamp: 'kph 2/13/2007 21:44'!
nextParametersOldStyle

	"this utility extracts the unprocessed parameters into a dictionary from the typical old style form:
	
	KEY value Key2 Value Key3 Value :NextLauncherClass KEY value Key2 value etc.
	
	the ability to specify the nex launcher class by beginning a parameter 
	with a non letter character has been added.
	"

	| key value parameters |
	
	parameters := Dictionary new.
	
	parameters at: #launcher put: launchClass.
 
	[ (key := image getSystemAttribute: nextParameterIndex) isEmptyOrNil] 
		whileFalse:[
			nextParameterIndex := nextParameterIndex + 1.
			key first isLetter ifFalse: [ 
				nextLaunchDocument := key copyWithoutFirst. 
				^ parameters ].
			
			value := image getSystemAttribute: nextParameterIndex.
			nextParameterIndex := nextParameterIndex + 1.
			
			parameters at: key asUppercase put: value. 
	].

	^parameters

			
			
			
			! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 08:04'!
parameters: dict

"called from AutoStart in order to give us our parameters, we handle our parameters ourself so we can ignore this"! !

!Launcher methodsFor: 'reporting' stamp: 'kph 2/14/2007 11:45'!
report: reportSelector for: item on: stream
	
	(item respondsTo: reportSelector) ifFalse: [ self error: 'requested report not available' ].
	
	(item perform: reportSelector) ifNotNilDo: [ :result | 
		^result isString ifTrue: [ result printOn: stream ] 
		 			   ifFalse: [ result do: [ :each | each printOn: stream. stream cr ] ].
	].
	
	
	 ! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 07:16'!
startParameters

^self isStart ifTrue: [ self nextParametersNewStyle ] ifFalse: [ Dictionary new ]! !

!Launcher methodsFor: 'public' stamp: 'kph 2/14/2007 08:08'!
startUp 

"we are sent this when starting from autostart"

self beLauncher;
 launchImage: SmalltalkImage current withClass: self class
! !

!Launcher methodsFor: 'reporting' stamp: 'kph 2/15/2007 05:15'!
stdout

^Script new stdout! !

Object subclass: #Script
	instanceVariableNames: 'stderr stdin stdout'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Launcher-Core'!

!Script class methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 00:52'!
launchFrom: launcher

	^self launchWith: launcher launchParameters! !

!Script class methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 03:51'!
launchHelp

^
'+help,h      Prints this help text.
eval, e=     Evaluate code and quit.
-quit       Remain open after evaluating code'! !

!Script class methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 04:01'!
launchWith: params

| script quit |
params at: 'E' ifPresent: [:v | params at: 'EVAL' put: v ].
params at: 'P' ifPresent: [:v | params at: 'PRINT' put: v ].
params at: 'H' ifPresent: [:v | params at: 'HELP' put: v ].

script := self new.

params at: 'HELP' ifPresent: [ :v | 
	 script writeHelp.
].

params at: 'EVAL' ifPresent: [ :v | 
	 script evaluate: v.
].

params at: 'PRINT' ifPresent: [ :v | 
	 script evaluateAndPrint: v.
].

quit := params at: 'QUIT' ifAbsent: true.

quit ~= false ifTrue: [ SmalltalkImage current snapshot: false andQuit: true ].

^true! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 04:09'!
evaluate: code

	^Compiler new evaluate: code in: nil to: self.! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 04:10'!
evaluateAndPrint: code

stdout nextPutAll: (self evaluate: code) asString; cr
	 ! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 05:39'!
initialize

stdout := (MultiByteFileStream new) 
			wantsLineEndConversion: #crlf; 
			open: '/dev/stdout' forWrite: true.
			 
			
stderr :=  (MultiByteFileStream new) 
			wantsLineEndConversion: #crlf; 
			open: '/dev/stderr' forWrite: true. 
			 
stdin  := AsyncFile new open: '/dev/stdin' forWrite: false.! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 04:00'!
print: code

	 ! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 05:14'!
stderr

^stderr! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 05:13'!
stdout

^stdout! !

!Script methodsFor: 'as yet unclassified' stamp: 'kph 2/15/2007 02:22'!
writeHelp

	| helps |
	
	helps := SystemNavigation default allImplementorsOf: #launchHelp.
	
	helps do: [ :mr | 
		stdout nextPutAll: mr classSymbol; cr;cr.
		stdout nextPutAll: (mr actualClass soleInstance launchHelp); cr;cr.
	].! !

!ProjectLauncher methodsFor: '*launcher-core' stamp: 'kph 2/15/2007 02:28'!
startUpAfterLogin
	| scriptName loader isUrl |
	self setupFlaps.
	Preferences readDocumentAtStartup ifTrue: [
		HTTPClient isRunningInBrowser ifTrue:[
			self setupFromParameters.
			scriptName _ self parameterAt: 'src'.
			CodeLoader defaultBaseURL: (self parameterAt: 'Base').
		] ifFalse:[
			scriptName _ (SmalltalkImage current getSystemAttribute: 2) ifNil:[''].
			scriptName _ scriptName convertFromSystemString.
			scriptName isEmpty ifFalse:[
				"figure out if script name is a URL by itself"
				isUrl _ (scriptName asLowercase beginsWith:'http://') or:[
						(scriptName asLowercase beginsWith:'file://') or:[
						(scriptName asLowercase beginsWith:'ftp://')]].
				isUrl ifFalse:[
					Smalltalk at: scriptName asSymbol
					ifPresent: [:launchClass | ^ Launcher newLauncher initiateStartingSequence ].
					
					scriptName _ 'file:', (FileDirectory default fullNameFor: scriptName) ]
				].
		]. ]
	ifFalse: [ scriptName := '' ].

	scriptName isEmptyOrNil
		ifTrue:[^Preferences eToyFriendly ifTrue: [self currentWorld addGlobalFlaps]].
	loader _ CodeLoader new.
	loader loadSourceFiles: (Array with: scriptName).
	(scriptName asLowercase endsWith: '.pr') 
		ifTrue:[self installProjectFrom: loader]
		ifFalse:[loader installSourceFiles].
! !

!Installer class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:22'!
launchFrom: launcher

	^self launchWith: launcher launchParameters! !

!Installer class methodsFor: '*launcher-core' stamp: 'kph 2/15/2007 05:24'!
launchWith: params

	|  skipTestsFlag |

 	params at: 'P' ifPresent: [ :v | params at: 'PATH' put: v ].
 	params at: 'I' ifPresent: [ :v | params at: 'INSTALL' put: v ].
 	params at: 'IN' ifPresent: [ :v | params at: 'INSTALL' put: v ].
 	params at: 'U' ifPresent: [ :v | params at: 'URL' put: v ].

	params at: 'PATH' ifPresent: [ :v | 
		self webSearchPathFrom: v.
	].

	params at: 'VIEW' ifPresent: [ :v |
		self view: v
	].

	IsSetToTrapErrors := true.
	params at: 'DEBUG' ifPresent: [ :v | IsSetToTrapErrors := (v == true) not ].

	skipTestsFlag := false.
	params at: 'SKIPTESTS' ifPresent: [ :v | skipTestsFlag := true ].

 	params at: 'URL' ifPresent: [ :v | 
		  self skipLoadingTests: skipTestsFlag during: [ self installUrl: v] ].
 
 	params at: 'INSTALL' ifPresent: [ :v | 
		  self skipLoadingTests: skipTestsFlag during: [ self install: v] ].
 
	 ^true

	! !

!WAKom class methodsFor: '*launcher-core' stamp: 'kph 2/15/2007 03:46'!
launchFrom: launcher

^ self launchWith: launcher launchParameters! !

!WAKom class methodsFor: '*launcher-core' stamp: 'kph 2/15/2007 03:56'!
launchHelp

^
'+stop      Stops the WAKom server
start=<p>  Starts the server on the given port'! !

!WAKom class methodsFor: '*launcher-core' stamp: 'kph 2/15/2007 03:57'!
launchWith: params

params at: 'STOP' ifPresent: [ :v | self stop ].

params at: 'START' ifPresent: [ :v | self startOn: v asInteger ].

^true! !

!SystemNavigation class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:12'!
launchFrom: launcher

^launcher launchReportFor: self default! !

!SystemNavigation methodsFor: '*launcher-core' stamp: 'kph 2/3/2007 02:54'!
printCategoriesAndClasses

	^SystemOrganization printString
	
	 ! !

!SmalltalkImage class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 10:03'!
launchFrom: launcher

 ^self launchWith: launcher launchParameters! !

!SmalltalkImage class methodsFor: '*launcher-core' stamp: 'kph 2/14/2007 11:26'!
launchWith: params

	| saveAs quit resuming |

	saveAs := params at: 'SAVE' ifAbsent: false.
	quit := params at: 'QUIT' ifAbsent: false.

	(saveAs isKindOf: Boolean) 
		ifFalse: [ resuming := self current saveAs: saveAs.
				  resuming ifTrue: [ ^false ].
				  saveAs := false ].
		
 	resuming := self current snapshot: saveAs andQuit: (quit == true).
	 
	resuming ifTrue: [ ^false ].
	"returning false, stops evaluation of further commandline parameters
	 this will be beneficial for image restart"

	^ true
	! !
Launcher initialize!
